import icemac.addressbook.browser.interfaces
import textwrap
import z3c.flashmessage.receiver
import zope.component
import zope.contentprovider.provider
import zope.interface


@zope.component.adapter(
    zope.interface.Interface,
    icemac.addressbook.browser.interfaces.IAddressBookLayer,
    zope.interface.Interface)
class MessagesContentProvider(
        zope.contentprovider.provider.ContentProviderBase,
        z3c.flashmessage.receiver.GlobalMessageReceiver):
    """Content provider displaying flash messages."""

    template = zope.browserpage.viewpagetemplatefile.ViewPageTemplateFile(
        'messages.pt')
    message_display_timeout = 3000  # ms

    def update(self):
        self.messages = list(self.receive())

    def render(self):
        if self.messages:
            return self.template(self)
        return ''

    def js(self):
        return textwrap.dedent('''
            $(document).ready(function() {
                setTimeout(
                    function() {
                        $("#info-messages").fadeOut("normal")
                    },
                    %d);
                })
        ''' % self.message_display_timeout)



from icemac.addressbook.i18n import _
from js.cookieconsent import cookieconsent
import json
import os
import zope.i18n
import zope.viewlet.viewlet


class CookieConsentViewlet(zope.viewlet.viewlet.ViewletBase):
    """Configure and render the cookieconsent dialog."""

    def _t(self, message_id):
        return zope.i18n.translate(message_id, context=self.request)

    def update(self):
        cookieconsent.need()
        self.config = {
            "palette": {
                "popup": {
                    "background": "#383b75"
                },
                "button": {
                    "background": "#f1d600"
                }
            },
            "theme": "classic",
            "position": "bottom-right",
            "content": {
                "message": self._t(
                    _("This web application uses a session cookie to"
                      " store the log-in status. It gets deleted when"
                      " closing the browser.")),
                "dismiss": self._t(_("I understand!")),
                "href": None,
                "link": None,
            }
        }
        url = os.environ.get('AB_LINK_DATAPROTECTION_URL', None)
        if url is not None:
            self.config['content']['href'] = url
            self.config['content']['link'] = self._t(_("Read more."))

    def render(self):
        return u'''\
            <script>
                window.addEventListener("load", function(){
                window.cookieconsent.initialise(%s)});
            </script>''' % json.dumps(self.config)



from icemac.addressbook.interfaces import IEntityOrder
from icemac.addressbook.interfaces import IKeywordTitles, ITitle, IPhoneNumber
from icemac.addressbook.interfaces import IPerson, IPersonDefaults, IEntity
from icemac.addressbook.interfaces import ISchemaName
from icemac.addressbook.person import Person, Keywords
import gocept.reference.verify
import zope.component


def test_person__Person__1():
    """It fulfils the `IPerson` interface."""
    assert gocept.reference.verify.verifyObject(IPerson, Person())


def test_person__Person__2():
    """It fulfils the `IPersonDefaults` interface."""
    assert gocept.reference.verify.verifyObject(IPersonDefaults, Person())


def test_person__Person__schema__1(zcmlS):
    """It can be adapted to ``ISchemaName``."""
    assert 'IPerson' == ISchemaName(Person()).schema_name


def test_person__Keywords__1():
    """It fulfills the `IKeywordTitles` interface."""
    assert gocept.reference.verify.verifyObject(IKeywordTitles, Keywords(None))


def test_person__PersonDefaultsEntity__getRawFields__1(address_book):
    """It returns the fields in default sort order if no order is defined."""
    entity = IEntity(IPersonDefaults)
    assert ([
        'default_postal_address',
        'default_phone_number',
        'default_email_address',
        'default_home_page_address',
    ] == [x[0] for x in entity.getRawFields()])


def test_person__PersonDefaultsEntity__getRawFields__2(address_book):
    """It returns the fields in the entity sort order."""
    entity = IEntity(IPersonDefaults)
    zope.component.getUtility(IEntityOrder).up(IEntity(IPhoneNumber))
    assert ([
        'default_phone_number',
        'default_postal_address',
        'default_email_address',
        'default_home_page_address',
    ] == [x[0] for x in entity.getRawFields()])


def test_person__PersonDefaultsEntity__getRawFields__3(address_book):
    """It returns the fields in default order if called whih `sorted=False`."""
    entity = IEntity(IPersonDefaults)
    zope.component.getUtility(IEntityOrder).up(IEntity(IPhoneNumber))
    assert ([
        'default_postal_address',
        'default_phone_number',
        'default_email_address',
        'default_home_page_address',
    ] == [x[0] for x in entity.getRawFields(sorted=False)])


def test_person__Person__get_name__1():
    """It returns last name and first name if both are set."""
    person = Person()
    person.first_name = u'Bernd'
    person.last_name = u'Tester'
    assert u'Bernd Tester' == person.get_name()


def test_person__Person__get_name__2():
    """It returns the last name if the first name is not set."""
    person = Person()
    person.last_name = u'Tester'
    assert u'Tester' == person.get_name()


def test_person__Person__get_name__3():
    """It returns the first name if the last name is not set."""
    person = Person()
    person.first_name = u'Berns'
    assert u'Berns' == person.get_name()


def test_person__Person__get_name__4():
    """It returns '' if neither first name nor last name is set."""
    assert u'' == Person().get_name()


def test_person__title__1(zcmlS):
    """The title of a person without a name is a fix string."""
    assert u'<no name>' == ITitle(Person())


def test_person__title__2(zcmlS):
    """`title()` returns the last name if the first name is empty."""
    person = Person()
    person.last_name = u'Tester'
    assert u'Tester' == ITitle(person)


def test_person__title__3(zcmlS):
    """`title()` returns the last name and first name comma separated."""
    person = Person()
    person.first_name = u'Hans'
    person.last_name = u'Tester'
    assert u'Tester, Hans' == ITitle(person)



import collections
import z3c.form.browser.select
import z3c.form.interfaces


# The used JS functions on `document` are defined in ../resources/js/form.js
JAVASCRIPT_TEMPLATE = '''\
$(document).ready(function(){
    $('#%(id)s').select2({
        width: '35em'
    });
    // handle ordering at select and make element sortable:
    $('#%(id)s').on('select2:select',
                    document.select2__keep_selected_items_ordered);
    document.select2__make_selected_items_sortable($('#%(id)s'));
});
'''


class Select2ListFieldWidget(z3c.form.browser.select.SelectWidget):
    """Widget for a list field rendered using the select2 JS library."""

    klass = u'select2-ordered-multi-select'
    multiple = 'multiple'

    @property
    def items(self):
        items = super(Select2ListFieldWidget, self).items
        value_item_map = collections.OrderedDict(
            (x['value'], x) for x in items)
        # Move the selected items to the top to keep the order like JS does it:
        not_selected_items = [data
                              for value, data in value_item_map.items()
                              if value not in self.value]
        selected_items = [value_item_map[x]
                          for x in self.value]
        return selected_items + not_selected_items

    def javascript(self):
        """Return JavaScript code for the widget."""
        return JAVASCRIPT_TEMPLATE % {'id': self.id}


def Select2ListChoiceFieldWidgetFactory(context, field, request):
    """Create a Select2ListFieldWidget as field widget.

    This factory is registered for list of choice.
    """
    return z3c.form.widget.FieldWidget(
        context, Select2ListFieldWidget(request))



import grokcore.component as grok
import icemac.addressbook.interfaces
import icemac.addressbook.principals.interfaces
import six
import zope.component.hooks
import zope.interface
import zope.interface.interfaces
import zope.schema.interfaces
import zope.security.proxy


@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def gocept_country_title(obj):
    """Title for objects from gocept.country."""
    return obj.name


@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def obj_dot_title(obj):
    """Title for an `obj` where the title is stored on `title` attribute."""
    return obj.title or u''


@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
def default_title(obj):
    """Default title adapter which returns `str` representation of `obj`."""
    if isinstance(obj, six.string_types):
        return obj
    return str(obj)


@grok.adapter(Exception)
@grok.implementer(icemac.addressbook.interfaces.ITitle)
def exception_title(exc):
    """Special exception title to prevent lengthy Chameleon output."""
    return u"<{0.__class__.__name__}{0.args}>".format(exc)


@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(icemac.addressbook.principals.interfaces.IRoot)
def principal_root(principal):
    """Return the site root."""
    return zope.component.hooks.getSite()


@grok.implementer(icemac.addressbook.interfaces.ISchemaName)
class SchemaProvider_SchemaName(grok.Adapter):
    """Adapter between ISchemaProvider and ISchemaName."""

    grok.context(icemac.addressbook.interfaces.ISchemaProvider)

    def __init__(self, context):
        # Instead of removing the security proxy I could require a ZCML class
        # definition of the usable interfaces. They are not writeable so it
        # seems too much effort to require the declaration.
        self.schema_name = unicode(
            zope.security.proxy.getObject(context.schema).getName())


@grok.implementer(icemac.addressbook.interfaces.ISchemaName)
class Interface_SchemaName(grok.Adapter):
    """Adapter between Interface and ISchemaName."""

    grok.context(zope.interface.interfaces.IInterface)

    def __init__(self, context):
        self.schema_name = unicode(context.getName())



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.addressbook.browser.base
import icemac.addressbook.browser.breadcrumb
import icemac.addressbook.browser.menus.menu
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import six
import z3c.flashmessage.interfaces
import z3c.table.column
import zope.container.traversal
import zope.location
import zope.publisher.interfaces.http
import zope.security.proxy
import zope.traversing.browser.absoluteurl


class EntitiesBreadCrumb(
        icemac.addressbook.browser.breadcrumb.MasterdataChildBreadcrumb):
    """Breadcrumb for the entities."""

    grok.adapts(
        icemac.addressbook.interfaces.IEntities,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Entities')


class UpLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """Column displaying an `up` link."""

    header = _('move-up-table-header', default='move')
    linkName = 'up.html'
    linkContent = _('move-up-table-cell', default='up')

    def renderCell(self, item):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        if order.isFirst(item):
            return u''
        return super(UpLinkColumn, self).renderCell(item)


class DownLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """Column displaying an `down` link."""

    header = _('move-down-table-header', default='move')
    linkName = 'down.html'
    linkContent = _('move-down-table-cell', default='down')

    def renderCell(self, item):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        if order.isLast(item):
            return u''
        return super(DownLinkColumn, self).renderCell(item)


class EditFieldsLinkColumn(icemac.addressbook.browser.table.LinkColumn):
    """LinkColumn only displaying a link for IMayHaveUserFields items."""

    linkContent = _(u'Edit fields')
    header = u''

    def renderCell(self, item):
        if icemac.addressbook.interfaces.IEditableEntity.providedBy(item):
            return super(EditFieldsLinkColumn, self).renderCell(item)
        return ''


class List(icemac.addressbook.browser.table.Table):
    """List existing entities."""

    title = icemac.addressbook.browser.breadcrumb.DO_NOT_SHOW
    sortOn = None  # do not sort rows

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, z3c.table.column.I18nGetAttrColumn, 'title', weight=10,
                header=_(u'Entity'), attrName='title'),
            z3c.table.column.addColumn(
                self, UpLinkColumn, 'up', weight=20),
            z3c.table.column.addColumn(
                self, DownLinkColumn, 'down', weight=30),
            z3c.table.column.addColumn(
                self, EditFieldsLinkColumn, 'fields', weight=100)]

    @property
    def values(self):
        # Need to remove security otherwise local administrators get
        # an unauthorized exception.
        for x in self.context.getEntities():
            yield zope.security.proxy.getObject(x)


@zope.component.adapter(
    icemac.addressbook.interfaces.IEntity,
    zope.publisher.interfaces.http.IHTTPRequest)
class EntityAbsoluteURL(zope.traversing.browser.absoluteurl.AbsoluteURL):
    """AbsoluteURL adapter for an entity."""

    def __str__(self):
        # parent is the entities utility and name is the class name
        parent = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        url = str(zope.component.getMultiAdapter(
            (parent, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL))
        url += '/' + self.context.class_name
        return url


@zope.component.adapter(
    icemac.addressbook.interfaces.IEntities,
    zope.publisher.interfaces.http.IHTTPRequest)
@zope.interface.implementer_only(zope.publisher.interfaces.IPublishTraverse)
class EntitiesTraverser(zope.container.traversal.ItemTraverser):
    """Make entities located and traverable."""

    def publishTraverse(self, request, name):
        entity = zope.component.queryUtility(
            icemac.addressbook.interfaces.IEntity, name=name)
        if entity is not None:
            return zope.location.LocationProxy(entity, self.context, name)
        return super(EntitiesTraverser, self).publishTraverse(request, name)


class MoveBase(icemac.addressbook.browser.base.BaseView):
    """Base class for movement views."""

    direction = None  # set on sub-class

    def __call__(self):
        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        getattr(order, self.direction)(self.context)
        self.request.response.redirect(self.url(self.context.__parent__))
        message = _(six.text_type(self.message),
                    mapping=dict(entity=self.context.title))
        zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource).send(message)


class MoveUp(MoveBase):
    """Move entity up in entity order."""

    direction = 'up'
    message = _('Moved ${entity} up.')


class MoveDown(MoveBase):
    """Move entity down in entity order."""

    direction = 'down'
    message = _('Moved ${entity} down.')


entity_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    icemac.addressbook.interfaces.IEntities,
    icemac.addressbook.interfaces.IEntity,
    icemac.addressbook.interfaces.IField)
